@@ -27,10 +27,8 @@ open Sig
 module T = Term
 module F = Formula
 module MF = F.Map
-module SF = F.Set
 module Ex = Explanation
 module MT = T.Map
-module ST = T.Set
 module SubstT = Term.Subst
 
 type gsubst = {
@@ -42,11 +40,10 @@ type gsubst = {
 		     avec le but de la PO *)
   s_term_orig : T.t list;
   s_lem_orig : F.t;
-  trs : T.t list;
 }
 
 type trigger_info = {
-  trigger_query : Literal.LT.t option ;
+  trigger : F.trigger;
   trigger_age : int ;  (* age d'un trigger *)
   trigger_orig : F.t ; (* lemme d'origine *)
   trigger_formula : F.t ; (* formule associee au trigger *)
@@ -60,19 +57,46 @@ type term_info = {
   term_from_terms : T.t list;
 }
 
-module EMatch (X : Theory.S) = struct
+type info = {
+  age : int ; (* age du terme *)
+  lem_orig : F.t list ; (* lemme d'ou provient eventuellement le terme *)
+  t_orig : T.t list;
+  but : bool  (* le terme a-t-il un lien avec le but final de la PO *)
+}
 
-  type info = {
-    age : int ; (* age du terme *)
-    lem_orig : F.t list ; (* lemme d'ou provient eventuellement le terme *)
-    t_orig : T.t list;
-    but : bool  (* le terme a-t-il un lien avec le but final de la PO *)
-  }
+module type S = sig
+  type t
+  type theory
+
+  val empty : t
+  val add_term : term_info -> Term.t -> t -> t
+  val max_term_depth : t -> int -> t
+  val add_triggers : t -> (int * Explanation.t) Formula.Map.t -> t
+  val terms_info : t -> info Term.Map.t
+  val query : t -> theory -> (trigger_info * gsubst list) list
+  val unused_context : Formula.t -> bool
+
+end
+
+
+module type Arg = sig
+  type t
+  val term_repr : t -> Term.t -> Term.t
+  val add_term : t -> Term.t -> t
+  val are_equal : t -> Term.t -> Term.t -> add_terms:bool -> Sig.answer
+  val class_of : t -> Term.t -> Term.t list
+end
+
+
+module Make (X : Arg) : S with type theory = X.t = struct
+
+  type theory = X.t
 
   type t = {
     fils : T.t list MT.t SubstT.t ;
     info : info MT.t ;
-    pats : (trigger_info * T.t list) list
+    max_t_depth : int;
+    pats : trigger_info list
   }
 
   exception Echec
@@ -81,6 +105,7 @@ module EMatch (X : Theory.S) = struct
     fils = SubstT.empty ;
     info = MT.empty ;
     pats = [ ];
+    max_t_depth = 0;
   }
 
   let age_limite = Options.age_bound
@@ -91,19 +116,18 @@ module EMatch (X : Theory.S) = struct
   module Debug = struct
 
     let add_term t =
-      if debug_matching() then
+      if debug_matching() >= 3 then
         fprintf fmt "[matching] add_term:  %a@." T.print t
 
-    let matching pats =
-      if debug_matching() then begin
-        fprintf fmt "@.[matching] (multi-)trigger: ";
-        List.iter (fprintf fmt "%a , " T.print) pats;
-        fprintf fmt "@.";
+    let matching tr =
+      if debug_matching() >= 3 then begin
+        fprintf fmt "@.[matching] (multi-)trigger: %a@."
+          T.print_list tr.F.content;
         fprintf fmt "========================================================@."
       end
 
     let match_pats_modulo pat lsubsts =
-      if debug_matching() then begin
+      if debug_matching() >= 3 then begin
         fprintf fmt "@.match_pat_modulo: %a  with accumulated substs@."
           T.print pat;
         List.iter (fun {sbs=sbs; sty=sty} ->
@@ -113,13 +137,13 @@ module EMatch (X : Theory.S) = struct
       end
 
     let match_one_pat {sbs=sbs; sty=sty} pat0 =
-      if debug_matching() then
+      if debug_matching() >= 3 then
         fprintf fmt "@.match_pat: %a  with subst:  sbs= %a | sty= %a @."
           T.print pat0 (SubstT.print Term.print) sbs Ty.print_subst sty
 
 
     let match_one_pat_against {sbs=sbs; sty=sty} pat0 t =
-      if debug_matching() then
+      if debug_matching() >= 3 then
         fprintf fmt
           "@.match_pat: %a  against term %a@.with subst:  sbs= %a | sty= %a @."
           T.print pat0
@@ -128,13 +152,13 @@ module EMatch (X : Theory.S) = struct
           Ty.print_subst sty
 
     let match_term {sbs=sbs; sty=sty} t pat =
-      if debug_matching() then
+      if debug_matching() >= 3 then
         fprintf fmt
           "[match_term] I match %a against %a with subst: sbs=%a | sty= %a@."
           T.print pat T.print t (SubstT.print Term.print) sbs Ty.print_subst sty
 
     let match_list {sbs=sbs; sty=sty} pats xs =
-      if debug_matching() then
+      if debug_matching() >= 3 then
         fprintf fmt
           "@.[match_list] I match %a against %a with subst: sbs=%a | sty= %a@."
           T.print_list pats
@@ -143,10 +167,26 @@ module EMatch (X : Theory.S) = struct
           Ty.print_subst sty
 
     let match_class_of t cl =
-      if debug_matching() then
+      if debug_matching() >= 3 then
         fprintf fmt "class_of (%a)  = { %a }@."
           T.print t
           (fun fmt -> List.iter (fprintf fmt "%a , " T.print)) cl
+
+    let candidate_substitutions pat_info res =
+      if debug_matching() >= 1 then begin
+        fprintf fmt "[Matching.matching]@.";
+        fprintf fmt "%3d candidate substitutions for Axiom %a with trigger %a@."
+          (List.length res)
+          F.print pat_info.trigger_orig
+          T.print_list pat_info.trigger.F.content;
+        if debug_matching() >= 2 then
+          List.iter
+            (fun gsbt ->
+              fprintf fmt " >>> sbs = %a  and  sbty = %a@."
+                (SubstT.print T.print) gsbt.sbs Ty.print_subst gsbt.sty
+            )res
+
+      end
   end
   (*BISECT-IGNORE-END*)
 
@@ -193,13 +233,13 @@ module EMatch (X : Theory.S) = struct
     in
     if info.term_age > age_limite () then env else add_rec env t
 
-  let add_trigger p trs env = { env with pats = (p, trs) ::env.pats }
+  let add_trigger p env = { env with pats = p :: env.pats }
 
   let all_terms
-      f ty env pinfo
+      f ty env tbox
       {sbs=s_t; sty=s_ty; gen=g; goal=b;
        s_term_orig=s_torig;
-       s_lem_orig = s_lorig; trs=trs} lsbt_acc =
+       s_lem_orig = s_lorig} lsbt_acc =
     SubstT.fold
       (fun k s l ->
 	MT.fold
@@ -212,23 +252,60 @@ module EMatch (X : Theory.S) = struct
 		  max ng g , bt || b
 		with Not_found -> g , b
 	      in
+              (* with triggers that are variables, always normalize substs *)
+              let t = X.term_repr (X.add_term tbox t) t in
 	      { sbs = SubstT.add f t s_t;
 		sty = s_ty;
 		gen = ng;
 		goal = but;
 		s_term_orig = t :: s_torig;
 		s_lem_orig = s_lorig;
-                trs = trs
 	      }::l
 	    with Ty.TypeClash _ -> l
 	  ) s l
       ) env.fils lsbt_acc
 
-  let add_msymb tbox f t ({sbs=s_t; sty=s_ty} as sg)=
-    try
-      if X.are_equal tbox t (SubstT.find f s_t) = Sig.No then raise Echec;
+
+  module T2 = struct
+    type t = T.t * T.t
+    let compare (a, b) (x, y) =
+      let c = T.compare a x in if c <> 0 then c else T.compare b y
+  end
+
+  module MT2 = Map.Make(T2)
+
+  let wrap_are_equal_generic tbox t s add_terms cache_are_eq_gen =
+    try MT2.find (t, s) !cache_are_eq_gen
+    with Not_found ->
+      let res = X.are_equal tbox t s ~add_terms:add_terms in
+      cache_are_eq_gen :=
+        MT2.add (t, s) res (MT2.add (s, t) res !cache_are_eq_gen);
+      res
+
+  (* These references are reset before and after each call to query.
+     If some intermediate functions are exported in the future, the code
+     should be adapted. *)
+  let cache_are_equal_light = ref MT2.empty
+  let cache_are_equal_full  = ref MT2.empty
+
+  let are_equal_light tbox t s =
+    wrap_are_equal_generic tbox t s false cache_are_equal_light
+
+  let are_equal_full tbox t s =
+    wrap_are_equal_generic tbox t s true cache_are_equal_full
+
+  let add_msymb tbox f t ({sbs=s_t} as sg) max_t_depth =
+    if SubstT.mem f s_t then
+      let s = SubstT.find f s_t in
+      if are_equal_full tbox t s == Sig.No then raise Echec;
       sg
-    with Not_found ->  {sg with sbs=SubstT.add f t s_t; sty=s_ty}
+    else
+      let t =
+        if (T.view t).T.depth > max_t_depth || normalize_instances () then
+          X.term_repr (X.add_term tbox t) t
+        else t
+      in
+      {sg with sbs=SubstT.add f t s_t}
 
   let (-@) l1 l2 =
     match l1, l2 with
@@ -269,7 +346,7 @@ module EMatch (X : Theory.S) = struct
       in
       SLT.elements mtl
 
-  let rec match_term env tbox ( {sbs=s_t; sty=s_ty;gen=g;goal=b} as sg) pat t =
+  let rec match_term env tbox ({sty=s_ty;gen=g;goal=b} as sg) pat t =
     Options.exec_thread_yield ();
     Debug.match_term sg t pat;
     let {T.f=f_pat;xs=pats;ty=ty_pat} = T.view pat in
@@ -280,41 +357,41 @@ module EMatch (X : Theory.S) = struct
 	     let s_ty = Ty.matching s_ty ty_pat (T.view t).T.ty in
 	     let g',b' = infos max (||) t g b env in
 	     add_msymb tbox f_pat t
-	       { sg with sbs=s_t; sty=s_ty; gen=g'; goal=b' }
+	       { sg with sty=s_ty; gen=g'; goal=b' }
+               env.max_t_depth
 	   with Ty.TypeClash _ -> raise Echec)
         in
         [sb]
       | _ ->
         try
           let s_ty = Ty.matching s_ty ty_pat (T.view t).T.ty in
-          let cl = if no_Ematching () then [t] else X.class_of tbox t in
-          Debug.match_class_of t cl;
-          let cl =
-	    List.fold_left
-	      (fun l t ->
-                let {T.f=f; xs=xs; ty=ty} = T.view t in
-	        if Symbols.compare f_pat f = 0 then xs::l
-                else
-                  begin
-                    match f_pat, ty with
+          let gsb = { sg with sty = s_ty } in
+          if T.is_ground pat &&
+            are_equal_light tbox pat t != Sig.No then
+            [gsb]
+          else
+            let cl = if no_Ematching () then [t] else X.class_of tbox t in
+            Debug.match_class_of t cl;
+            let cl =
+	      List.fold_left
+	        (fun l t ->
+                  let {T.f=f; xs=xs; ty=ty} = T.view t in
+	          if Symbols.compare f_pat f = 0 then xs::l
+                  else
+                    begin
+                      match f_pat, ty with
                       | Symbols.Op (Symbols.Record), Ty.Trecord record ->
                         (xs_modulo_records t record) :: l
                       | _ -> l
-                  end
-	      )[] cl
-          in
-          let cl = filter_classes cl tbox in
-          let gsb = { sg with sbs = s_t; sty = s_ty } in
-          (**** This optim is very expensive***
-                if T.is_ground pat && are_equal tbox pat t <> Sig.No then
-                [gsb]
-                else*****)
-          (* pas sur que ce soit correct ici *)
-          List.fold_left
-            (fun acc xs ->
-              try (match_list env tbox gsb pats xs) -@ acc
-              with Echec -> acc
-            ) [] cl
+                    end
+	        )[] cl
+            in
+            let cl = filter_classes cl tbox in
+            List.fold_left
+              (fun acc xs ->
+                try (match_list env tbox gsb pats xs) -@ acc
+                with Echec -> acc
+              ) [] cl
         with Ty.TypeClash _ -> raise Echec
 
   and match_list env tbox sg pats xs =
@@ -331,12 +408,12 @@ module EMatch (X : Theory.S) = struct
         ) [sg] pats xs
     with Invalid_argument _ -> raise Echec
 
-  let match_one_pat env tbox pat_info pat0 lsbt_acc sg =
+  let match_one_pat env tbox pat0 lsbt_acc sg =
     Debug.match_one_pat sg pat0;
     let pat = T.apply_subst (sg.sbs, sg.sty) pat0 in
     let {T.f=f; xs=pats; ty=ty} = T.view pat in
     match f with
-      | Symbols.Var _ -> all_terms f ty env pat_info sg lsbt_acc
+      | Symbols.Var _ -> all_terms f ty env tbox sg lsbt_acc
       | _ ->
         let {sty=sty; gen=g; goal=b} = sg in
         let f_aux t xs lsbt =
@@ -356,11 +433,16 @@ module EMatch (X : Theory.S) = struct
 	try MT.fold f_aux (SubstT.find f env.fils) lsbt_acc
 	with Not_found -> lsbt_acc
 
-  let match_pats_modulo env tbox pat_info lsubsts pat =
+  let match_pats_modulo env tbox lsubsts pat =
     Debug.match_pats_modulo pat lsubsts;
-    List.fold_left (match_one_pat env tbox pat_info pat) [] lsubsts
+    List.fold_left (match_one_pat env tbox pat) [] lsubsts
 
-  let matching env tbox (pat_info, pats) =
+  let matching env tbox pat_info =
+    let pats = pat_info.trigger in
+    let pats_list =
+      List.stable_sort
+        (fun s t -> (T.view t).T.depth - (T.view s).T.depth) pats.F.content
+    in
     Debug.matching pats;
     let egs =
       { sbs = SubstT.empty;
@@ -369,13 +451,25 @@ module EMatch (X : Theory.S) = struct
 	goal = false;
 	s_term_orig = [];
 	s_lem_orig = pat_info.trigger_orig;
-        trs = pats;
       }
     in
-    pat_info, List.fold_left (match_pats_modulo env tbox pat_info) [egs] pats
+    let res = List.fold_left (match_pats_modulo env tbox) [egs] pats_list in
+    Debug.candidate_substitutions pat_info res;
+    pat_info, res
+
+  let reset_cache_refs () =
+    cache_are_equal_light := MT2.empty;
+    cache_are_equal_full  := MT2.empty
 
   let query env tbox =
-    List.rev_map (matching env tbox) env.pats
+    reset_cache_refs ();
+    try
+      let res = List.rev_map (matching env tbox) env.pats in
+      reset_cache_refs ();
+      res
+    with e ->
+      reset_cache_refs ();
+      raise e
 
   let query env tbox =
     if Options.timers() then
@@ -389,67 +483,29 @@ module EMatch (X : Theory.S) = struct
 	raise e
     else query env tbox
 
-end
+  let max_term_depth env mx = {env with max_t_depth = max env.max_t_depth mx}
 
-(*** Instantiation part of the module ***)
+  let add_triggers env formulas =
+    MF.fold
+      (fun lem (age, dep) env ->
+	match F.view lem with
+	| F.Lemma {F.triggers = tgs; main = f} ->
+	  List.fold_left
+	    (fun env tr ->
+	      let info =
+		{ trigger = tr;
+                  trigger_age = age ;
+		  trigger_orig = lem ;
+		  trigger_formula = f ;
+		  trigger_dep = dep}
+	      in
+              add_trigger info env
+            ) env tgs
 
-module type S = sig
-  type t
-  type tbox
+	| _ -> assert false
+      ) formulas env
 
-  val empty : t
-  val add_terms : t -> T.Set.t -> F.gformula -> t
-  val add_lemma : t -> F.gformula -> Ex.t -> t
-  val add_predicate : t -> F.gformula -> t
-
-  type instances = (F.gformula * Ex.t) list
-
-  val m_lemmas :
-    t ->
-    tbox ->
-    (F.t -> F.t -> bool) ->
-    instances * instances (* goal_directed, others *)
-
-  val m_predicates :
-    t ->
-    tbox ->
-    (F.t -> F.t -> bool) ->
-    instances * instances (* goal_directed, others *)
-
-  (* returns used axioms/predicates * unused axioms/predicates *)
-  val retrieve_used_context : t -> Ex.t -> Formula.t list * Formula.t list
-
-end
-
-module Make(X : Theory.S) : S with type tbox = X.t = struct
-
-  module EM = EMatch(X)
-
-  type tbox = X.t
-  type instances = (F.gformula * Ex.t) list
-
-  type t = {
-    lemmas : (int * Ex.t) MF.t;
-    predicates : (int * Ex.t) MF.t;
-    matching : EM.t;
-  }
-
-  let empty = {
-    lemmas = MF.empty ;
-    matching = EM.empty;
-    predicates = MF.empty;
-  }
-
-  let add_terms env s gf =
-    let infos = {
-      term_age = gf.F.age ;
-      term_from_goal    = gf.F.gf ;
-      term_from_formula = gf.F.lem ;
-      term_from_terms   = gf.F.from_terms
-    }
-    in
-    { env with
-      matching = T.Set.fold (EM.add_term infos) s env.matching }
+  let terms_info env = env.info
 
   module SST = Set.Make(String)
 
@@ -486,7 +542,7 @@ module Make(X : Theory.S) : S with type tbox = X.t = struct
     else None
 
   let parent s =
-    if s = "" then s
+    if String.length s = 0 then s
     else
       match s.[0] with
       | '#' ->
@@ -497,245 +553,7 @@ module Make(X : Theory.S) : S with type tbox = X.t = struct
   let unused_context f = match used, F.view f with
     | None  , _ -> false
     | Some s_used, F.Lemma {F.name=s} ->
-      not (s = "" || SST.mem (parent s) s_used)
+      not (String.length s = 0 || SST.mem (parent s) s_used)
     | _ -> assert false
 
-  let add_lemma env gf dep =
-    let {F.f=f;age=age} = gf in
-    if (*not (Ex.is_empty dep) ||*) unused_context f then env
-    else
-      let age , dep =
-        try
-          let age' , dep' = MF.find f env.lemmas in
-          min age age' , Ex.union dep dep'
-        with Not_found -> age , dep
-      in
-      { env with lemmas=MF.add f (age,dep) env.lemmas }
-
-  let add_predicate env gf =
-    let {F.f=f;age=age} = gf in
-    if unused_context f then env
-    else { env with predicates = MF.add f (age,Ex.empty) env.predicates }
-
-  let mtriggers env formulas =
-    MF.fold
-      (fun lem (age, dep) env ->
-	match F.view lem with
-	    F.Lemma {F.triggers = tgs; main = f} ->
-	      List.fold_left
-		(fun env (tg, a) ->
-		  let info =
-		    { trigger_age = age ;
-		      trigger_orig = lem ;
-		      trigger_formula = f ;
-		      trigger_dep = dep ;
-		      trigger_query = a }
-		  in
-		  { env with
-		    matching =
-		      EM.add_trigger info tg env.matching })
-		env tgs
-	  | _ -> assert false
-      )
-      formulas env
-
-  let record_this_instance accepted lorig =
-    if Options.profiling() then
-      match F.view lorig with
-      | F.Lemma {F.name;loc} -> Profiling.new_instance_of name loc accepted
-      | _ -> assert false
-
-  let profile_produced_terms env lorig nf s trs =
-    let st0 =
-      List.fold_left (fun st t -> T.subterms st (T.apply_subst s t))
-        T.Set.empty trs
-    in
-    let name, loc, f = match F.view lorig with
-      | F.Lemma {F.name;main;loc} -> name, loc, main
-      | _ -> assert false
-    in
-    let st1 = F.ground_terms_rec nf in
-    let diff = Term.Set.diff st1 st0 in
-    let info = env.EM.info in
-    let _new = Term.Set.filter (fun t -> not (MT.mem t info)) diff in
-    Profiling.register_produced_terms name loc st0 st1 diff _new
-
-  let new_facts env tbox selector substs =
-    List.fold_left
-      (fun acc ({trigger_formula=f; trigger_query = guard;
-	         trigger_age=age; trigger_dep=dep; trigger_orig=orig },
-	        subst_list) ->
-        List.fold_left
-	  (fun ((gd, ngd) as acc)
-	    {sbs = sbs;
-	     sty = sty;
-	     gen = g;
-	     goal = b;
-	     s_term_orig = torig;
-	     s_lem_orig = lorig;
-             trs=trs } ->
-              if not (F.equal orig lorig) then begin
-                fprintf fmt "orig =%a@.@." F.print orig;
-                fprintf fmt "lorig=%a@." F.print lorig;
-                assert false
-              end;
-              let sbs =
-                if Options.normalize_instances () then
-                  Symbols.Map.fold
-                    (fun k t mp ->
-                       Symbols.Map.add k (X.term_repr tbox t) mp
-                    )sbs Symbols.Map.empty
-                else sbs
-              in
-              let s = sbs, sty in
-	      match guard with
-	        | Some a when
-		    X.query (Literal.LT.apply_subst s a) tbox = No ->
-                  acc
-	        | _ ->
-		  let nf = F.apply_subst s f in
-                  let accepted = selector nf orig in
-                  record_this_instance accepted lorig;
-		  if accepted then
-		    let p =
-		      { F.f = nf;
-		        age = 1+(max g age);
-		        mf = true;
-		        gf = b;
-		        lem = Some lorig;
-		        from_terms = torig
-		      }
-                    in
-                    if Options.profiling() then
-                      profile_produced_terms env.matching lorig nf s trs;
-                    let dep =
-                      if not (Options.proof ()) then dep
-                      else Ex.union dep (Ex.singleton (Ex.Dep lorig))
-                    in
-                    if b then (* formula's terms are related to the goal *)
-                      (p,dep) :: gd, ngd
-                    else
-                      gd, (p,dep) :: ngd
-                  else acc
-	  )
-	  acc subst_list
-      )
-      ([], []) substs
-
-
-  let sort_facts =
-    let rec size f = match F.view f with
-      | F.Unit(f1,f2) -> max (size f1) (size f2)
-      | _             -> F.size f
-    in
-    fun lf ->
-      List.fast_sort
-        (fun (p1,_) (p2,_) ->
-          let c = size p1.F.f - size p2.F.f in
-          if c <> 0 then c
-          else F.compare p2.F.f p1.F.f
-        ) lf
-
-  let new_facts env tbox selector substs =
-    if Options.timers() then
-      try
-	Options.exec_timer_start Timers.M_Match Timers.F_new_facts;
-	let res = new_facts env tbox selector substs in
-	Options.exec_timer_pause Timers.M_Match Timers.F_new_facts;
-	res
-      with e ->
-	Options.exec_timer_pause Timers.M_Match Timers.F_new_facts;
-	raise e
-    else new_facts env tbox selector substs
-
-  let mround env axs tbox selector =
-    Options.tool_req 2 "TR-Sat-Mround";
-    let env = mtriggers env axs in
-    let substs = EM.query env.matching tbox in
-    let gd, ngd = new_facts env tbox selector substs in
-    sort_facts gd, sort_facts ngd
-
-  let m_lemmas env tbox selector = mround env env.lemmas tbox  selector
-
-  let m_predicates env tbox selector = mround env env.predicates tbox selector
-
-  module MI = Map.Make (struct type t = int let compare = compare end)
-
-  let retrieve_used_context env dep =
-    let deps = Ex.formulas_of dep in
-    let used, unlems, unpreds =
-      SF.fold
-        (fun f ((used, lems, preds) as acc) ->
-          if MF.mem f lems then f :: used, MF.remove f lems, preds
-          else if MF.mem f preds then f :: used, lems, MF.remove f preds
-          else acc
-        ) deps ([], env.lemmas, env.predicates)
-    in
-    let unused = MF.fold (fun f _ acc -> f::acc) unlems [] in
-    let unused = MF.fold (fun f _ acc -> f::acc) unpreds unused in
-    used, unused
-
-
-  (*** add wrappers to profile exported functions ***)
-
-  let add_terms env s gf =
-    if Options.timers() then
-      try
-	Options.exec_timer_start Timers.M_Match Timers.F_add_terms;
-	let res = add_terms env s gf in
-	Options.exec_timer_pause Timers.M_Match Timers.F_add_terms;
-	res
-      with e ->
-	Options.exec_timer_pause Timers.M_Match Timers.F_add_terms;
-	raise e
-    else add_terms env s gf
-
-  let add_lemma env gf dep =
-    if Options.timers() then
-      try
-	Options.exec_timer_start Timers.M_Match Timers.F_add_lemma;
-	let res = add_lemma env gf dep in
-	Options.exec_timer_pause Timers.M_Match Timers.F_add_lemma;
-	res
-      with e ->
-	Options.exec_timer_pause Timers.M_Match Timers.F_add_lemma;
-	raise e
-    else add_lemma env gf dep
-
-  let add_predicate env gf =
-    if Options.timers() then
-      try
-	Options.exec_timer_start Timers.M_Match Timers.F_add_predicate;
-	let res = add_predicate env gf in
-	Options.exec_timer_pause Timers.M_Match Timers.F_add_predicate;
-	res
-      with e ->
-	Options.exec_timer_pause Timers.M_Match Timers.F_add_predicate;
-	raise e
-    else add_predicate env gf
-
-  let m_lemmas env tbox selector =
-    if Options.timers() then
-      try
-	Options.exec_timer_start Timers.M_Match Timers.F_m_lemmas;
-	let res = m_lemmas env tbox selector in
-	Options.exec_timer_pause Timers.M_Match Timers.F_m_lemmas;
-	res
-      with e ->
-	Options.exec_timer_pause Timers.M_Match Timers.F_m_lemmas;
-	raise e
-    else m_lemmas env tbox selector
-
-  let m_predicates env tbox selector =
-    if Options.timers() then
-      try
-	Options.exec_timer_start Timers.M_Match Timers.F_m_predicates;
-	let res = m_predicates env tbox selector in
-	Options.exec_timer_pause Timers.M_Match Timers.F_m_predicates;
-	res
-      with e ->
-	Options.exec_timer_pause Timers.M_Match Timers.F_m_predicates;
-	raise e
-    else m_predicates env tbox selector
-
 end
