@@ -1,6 +1,6 @@
 (******************************************************************************)
 (*     Alt-Ergo: The SMT Solver For Software Verification                     *)
-(*     Copyright (C) 2013-2014 --- OCamlPro                                   *)
+(*     Copyright (C) 2013-2015 --- OCamlPro                                   *)
 (*     This file is distributed under the terms of the CeCILL-C licence       *)
 (******************************************************************************)
 
@@ -20,20 +20,21 @@
 (*   This file is distributed under the terms of the CeCILL-C licence         *)
 (******************************************************************************)
 
-open Why_ptree
+open Parsed
+open Typed
 
 type env
 
 val empty_env : env
 
-val file : bool -> env -> file -> 
+val file : bool -> env -> file ->
   ((int tdecl, int) annoted * env) list * env
 
-val split_goals : 
-  ((int tdecl, int) annoted * env) list -> 
+val split_goals :
+  ((int tdecl, int) annoted * env) list ->
   ((int tdecl, int) annoted * env) list list
 
-val term : env -> (Symbols.t * Ty.t) list -> Why_ptree.lexpr -> 
+val term : env -> (Symbols.t * Ty.t) list -> Parsed.lexpr ->
   (int tterm, int) annoted
 
 val new_id : unit -> int
