@@ -1,3 +1,9 @@
+(******************************************************************************)
+(*     Alt-Ergo: The SMT Solver For Software Verification                     *)
+(*     Copyright (C) 2013-2015 --- OCamlPro                                   *)
+(*     This file is distributed under the terms of the CeCILL-C licence       *)
+(******************************************************************************)
+
 (******************************************************************************)
 (*     The Alt-Ergo theorem prover                                            *)
 (*     Copyright (C) 2006-2013                                                *)
@@ -20,3 +26,4 @@ exception Progress
 exception NotCongruent
 exception Trivial
 exception Interpreted_Symbol
+exception Compared of int
