@@ -1,6 +1,6 @@
 (******************************************************************************)
 (*     Alt-Ergo: The SMT Solver For Software Verification                     *)
-(*     Copyright (C) 2013-2014 --- OCamlPro                                   *)
+(*     Copyright (C) 2013-2015 --- OCamlPro                                   *)
 (*     This file is distributed under the terms of the CeCILL-C licence       *)
 (******************************************************************************)
 
@@ -23,11 +23,11 @@
 module type S = sig
 
   (* the type of amalgamated AC semantic values *)
-  type r 
+  type r
 
   (* the type of AC semantic values used by the theory *)
   type t = r Sig.ac
-      
+
   (* builds an embeded semantic value from an AC term *)
   val make : Term.t -> r * Literal.LT.t list
 
@@ -37,6 +37,9 @@ module type S = sig
   (* compares two AC semantic values *)
   val compare : t -> t -> int
 
+  (* tests if two values are equal (using tags) *)
+  val equal : t -> t -> bool
+
   (* hash function for ac values *)
   val hash : t -> int
 
@@ -45,21 +48,21 @@ module type S = sig
 
   (* prints the AC semantic value *)
   val print : Format.formatter -> t -> unit
-    
+
   (* returns the leaves of the given AC semantic value *)
   val leaves : t -> r list
 
   (* replaces the first argument by the second one in the given AC value *)
   val subst : r -> r -> t -> r
 
-  (* add flatten the 2nd arg w.r.t HS.t, add it to the given list 
+  (* add flatten the 2nd arg w.r.t HS.t, add it to the given list
      and compact the result *)
   val add : Symbols.t -> r * int -> (r * int) list -> (r * int) list
 
   val fully_interpreted : Symbols.t -> bool
 
   val abstract_selectors : t -> (r * r) list -> r * (r * r) list
-    
+
   val compact : (r * int) list -> (r * int) list
 
 end
