@@ -35,13 +35,17 @@ module type S = sig
 
   val query : Literal.LT.t -> t -> Sig.answer
   val class_of : t -> Term.t -> Term.t list
-  val are_equal : t -> Term.t -> Term.t -> Sig.answer
+  val are_equal : t -> Term.t -> Term.t -> add_terms:bool -> Sig.answer
   val print_model : Format.formatter -> t -> unit
   val cl_extract : t -> Term.Set.t list
   val term_repr : t -> Term.t -> Term.t
   val extract_ground_terms : t -> Term.Set.t
   val get_real_env : t -> Ccx.Main.t
   val get_case_split_env : t -> Ccx.Main.t
+  val do_case_split : t -> t * Term.Set.t
+  val add_term : t -> Term.t -> add_in_cs:bool -> t
+
+  val compute_concrete_model : t -> t
 end
 
 module Main : S
