@@ -31,11 +31,12 @@ module M = struct
   let type_only = ref false
   let parse_only = ref false
   let steps_bound = ref (-1)
-  let age_bound = ref 10
+  let age_bound = ref 50
   let debug = ref false
   let debug_warnings = ref false
   let notriggers = ref false
   let debug_cc = ref false
+  let debug_gc = ref false
   let debug_use = ref false
   let debug_arrays = ref false
   let debug_uf = ref false
@@ -56,33 +57,42 @@ module M = struct
   let triggers_var = ref false
   let nb_triggers = ref 2
   let enable_assertions = ref false
-  let select = ref 0
   let rm_eq_existential = ref false
   let no_Ematching = ref false
+  let no_backjumping = ref false
   let nocontracongru = ref false
   let term_like_pp = ref true
   let debug_types = ref false
   let all_models = ref false
   let model = ref false
   let complete_model = ref false
+  let interpretation = ref 0
+  let debug_interpretation = ref false
   let proof = ref false
   let debug_proof = ref false
   let rules = ref (-1)
   let max_split = ref (Numbers.Q.from_int 1000000)
+  let fm_cross_limit = ref (Numbers.Q.from_int 10_000)
+  let case_split_policy = ref Util.AfterTheoryAssume
   let restricted = ref false
   let bottom_classes = ref false
   let timelimit = ref 0.
-  let debug_matching = ref false
+  let interpretation_timelimit = ref (if Sys.win32 then 0. else 1.)
+  let debug_matching = ref 0
+  let debug_explanations = ref false
   let sat_plugin = ref ""
   let inequalities_plugin = ref ""
   let profiling_plugin = ref ""
+  let cumulative_time_profiling = ref false
   let normalize_instances = ref false
   let partial_bmodel = ref true
   let tighten_vars = ref false
   let no_tcp = ref false
   let no_theory = ref false
   let js_mode = ref false
-  let backward_compat = ref false
+  let no_NLA = ref false
+  let no_ac = ref false
+  let instantiate_after_backjump = ref false
 
   let show_where s=
     match s with
@@ -112,6 +122,11 @@ module M = struct
       try Numbers.Q.from_string s
       with Failure _ -> Numbers.Q.m_one
 
+  let set_fm_cross_limit s =
+    fm_cross_limit :=
+      try Numbers.Q.from_string s
+      with Failure _ -> Numbers.Q.m_one
+
   let set_sat_plugin s = sat_plugin := s
 
   let set_inequalities_plugin s = inequalities_plugin := s
@@ -128,10 +143,11 @@ module M = struct
     | "arith" -> rules := 4
     | _ -> rules := -1
 
-  let set_limit t =
-    match Sys.os_type with
-      | "Win32" -> Format.eprintf "timelimit not supported on Win32 (ignored)@."
-      | _ -> timelimit := t
+  let set_limit timelimit_target t =
+    if Sys.win32 then
+      Format.eprintf "timelimit not supported on Win32 (ignored)@."
+    else
+      timelimit_target := t
 
   let replay = ref false
   let replay_used_context = ref false
@@ -147,6 +163,15 @@ module M = struct
     try profiling_period := float_of_string s
     with _ -> ()
 
+  let set_case_split_policy_option s =
+    case_split_policy :=
+      match s with
+      | "after-theory-assume" -> Util.AfterTheoryAssume
+      | "before-matching" -> Util.BeforeMatching
+      | "after-matching" -> Util.AfterMatching
+      | _ -> raise (Arg.Bad ("Bad value '"^s^"' for option -case-split-policy"))
+
+
   let timers = ref false
 
   let usage = "usage: alt-ergo [options] file.<why|mlw>"
@@ -161,6 +186,7 @@ module M = struct
     "-debug", Arg.Set debug, "  sets the debugging flag";
     "-dwarnings", Arg.Set debug_warnings, "  sets the debugging flag of warnings";
     "-dcc", Arg.Set debug_cc, "  sets the debugging flag of cc";
+    "-dgc", Arg.Set debug_gc, "  prints some debug info about the GC's activity";
     "-duse", Arg.Set debug_use, "  sets the debugging flag of use";
     "-duf", Arg.Set debug_uf, "  sets the debugging flag of uf";
     "-dfm", Arg.Set debug_fm, "  sets the debugging flag of inequalities";
@@ -179,7 +205,8 @@ module M = struct
     "-darrays", Arg.Set debug_arrays, "  sets the debugging flag of arrays";
     "-dcombine", Arg.Set debug_combine, "  sets the debugging flag of combine";
     "-dsplit", Arg.Set debug_split, "  sets the debugging flag of case-split analysis";
-    "-dmatching", Arg.Set debug_matching, "  sets the debugging flag of E-matching";
+    "-dmatching", Arg.Set_int debug_matching, "  sets the debugging flag of E-matching (0 = disabled, 1 = light, 2 = full)";
+    "-dexplanations", Arg.Set debug_explanations, "  sets the debugging flag of explanations";
     "-verbose", Arg.Set verbose, "  sets the verbose mode";
     "-version", Arg.Unit show_version, "  prints the version number";
     "-version-info", Arg.Unit show_version_info, "  prints some info about this version";
@@ -193,22 +220,32 @@ module M = struct
     " use all available ground terms in instanciation";
     "-nb-triggers" , Arg.Set_int nb_triggers,
     " number of redondant (multi)triggers (default: 2)";
-    "-select" , Arg.Set_int select,
-    "k tries to select relevant (at level k) hypotheses for each goal";
     "-triggers-var" , Arg.Set triggers_var , " allows variables as triggers";
     "-rm-eq-existential", Arg.Set rm_eq_existential, " substitute a variable in an existential when an equality gives the value of the variable";
     "-no-Ematching", Arg.Set no_Ematching, " disable matching modulo ground equalities";
+    "-no-backjumping", Arg.Set no_backjumping, " disable backjumping mechanism in DfsSAT";
+    "-no-NLA", Arg.Set no_NLA, " disable non-linear arithmetic reasoning (i.e. non-linear multplication, division and modulo on integers and rationals). Non-linear multiplication remains AC";
+    "-no-ac", Arg.Set no_ac, " Disable the AC theory of Associative and Commutative function symbols";
     "-nocontracongru", Arg.Set nocontracongru, "";
     "-term-like-pp", Arg.Set term_like_pp, " output semantic values as terms";
     "-all-models", Arg.Set all_models, " experimental support for all models";
     "-model", Arg.Set model, " experimental support for models on labeled terms";
     "-complete-model", Arg.Set complete_model, " experimental support for complete model";
+    "-dinterpretation", Arg.Set debug_interpretation, " set debug flag for interpretation generatation";
+    "-interpretation", Arg.Set_int interpretation, " experimental support for counter-example generation. Possible values are 1, 2, or 3 to compute an interpretation before returning Unknown, before instantiation, or before every decision or instantiation. A negative value (-1, -2, or -3) will disable interpretation display. Note that -max-split limitation will be ignored in model generation phase";
     "-proof", Arg.Set proof, " experimental support for succint proof";
     "-debug-proof", Arg.Set debug_proof, " replay unsatisfiable core produced by -proof. This options implies -proof";
     "-rules", Arg.String set_rules, "tr (tr in <parsing|typing|sat|cc|arith>) output rules used on stderr";
     "-max-split", Arg.String set_max_split,
     (Format.sprintf " maximum size of case-split (default value : %s)"
-       (Numbers.Q.to_string !max_split));
+                    (Numbers.Q.to_string !max_split));
+
+    "-fm-cross-limit", Arg.String set_fm_cross_limit,
+    (Format.sprintf " skip Fourier-Motzkin variables elimination steps that may produce a number of inequalities that is greater than the given limit (default value : %s). However, unit eliminations are always done" (Numbers.Q.to_string !fm_cross_limit));
+
+    "-case-split-policy", Arg.String set_case_split_policy_option,
+    " case-split policy. Set the case-split policy to use. Possible values are: after-theory-assume (default), before-matching, after-matching"
+    ;
     "-restricted", Arg.Set restricted,
     " restrict set of decision procedures (equality, arithmetic and AC)";
     "-bottom-classes", Arg.Set bottom_classes, " show equivalence classes at each bottom of the sat";
@@ -217,7 +254,8 @@ module M = struct
     "-replay-all-used-context", Arg.Set replay_all_used_context, " replay with all axioms and predicates saved in .used files of the current directory";
     "-save-used-context", Arg.Set save_used_context, " save used axioms and predicates in a .used file. This options implies -proof";
     "-replay-satml-dfs", Arg.Set replay_satml_dfs, " debug option for the satML plugin. Replays proven (valid) goals (with generated ground instances) using Dfs-sat";
-    "-timelimit", Arg.Float set_limit, "n set the time limit to n seconds (not supported on Windows)";
+    "-timelimit", Arg.Float (set_limit timelimit), "n set the time limit to n seconds (not supported on Windows)";
+    "-interpretation-timelimit", Arg.Float (set_limit interpretation_timelimit), "n set the time limit to n seconds for model generation (not supported on Windows). Default value is 1. sec";
     "-sat-plugin" , Arg.String set_sat_plugin,
     " use the given SAT-solver instead of the default DFS-based SAT solver";
     "-inequalities-plugin" , Arg.String set_inequalities_plugin,
@@ -225,11 +263,16 @@ module M = struct
     "-profiling", Arg.String parse_profiling, "<delay> activate the profiling module with the given frequency. Use Ctrl-C to switch between different views and \"Ctrl + AltGr + \\\" to exit.";
     "-profiling-plugin" , Arg.String set_profiling_plugin,
     " use the given profiling plugin";
+    "-cumulative-time-profiling", Arg.Set cumulative_time_profiling, " the time spent in called functions is also recorded in callers";
+
     "-rwt", Arg.Set rewriting, " use rewriting instead of axiomatic approach";
     "-normalize-instances" , Arg.Set normalize_instances,
-    " normalize generated ground instances w.r.t. the state of the theory. Default value: false";
-    "-backward-compat" , Arg.Set backward_compat,
-    " print Alt-Ergo's answer using old (<= 0.99.1) syntax. This option is useful when Alt-Ergo is used by tools that do not support new syntax yet.";
+    " normalize generated substitutions by matching w.r.t. the state of the theory. Default value is false. This means that only terms that are greater (w.r.t. depth) than the initial terms of the problem are normalized.";
+
+    "-inst-after-bj", Arg.Set instantiate_after_backjump,
+    " make a (normal) instantiation round after every backjump/backtrack"
+    ;
+
   ]
 
   let spec = Arg.align spec
@@ -242,7 +285,7 @@ module M = struct
     ref (fun _ _ -> ())
 
   let (timeout : (unit -> unit) ref) =
-    ref (fun () -> Format.printf "Timeout@."; exit 142)
+    ref (fun () -> raise Util.Timeout)
 
 end
 
@@ -250,8 +293,9 @@ let parse_args () =
   let ofile = ref None in
   let set_file s =
     if Filename.check_suffix s ".mlw" || Filename.check_suffix s ".why"
+       || Filename.check_suffix s ".zip"
     then ofile := Some s
-    else raise (Arg.Bad "no .mlw or .why extension")
+    else raise (Arg.Bad "no .mlw, .why or .zip extension")
   in
   Arg.parse M.spec set_file M.usage;
   match !ofile with
@@ -278,6 +322,7 @@ let _ = parse_args ()
 (** setters for debug flags *)
 let set_debug b = M.debug := b
 let set_debug_cc b = M.debug_cc := b
+let set_debug_gc b = M.debug_gc := b
 let set_debug_use b = M.debug_use := b
 let set_debug_uf b = M.debug_uf := b
 let set_debug_fm b = M.debug_fm := b
@@ -294,7 +339,8 @@ let set_debug_types b = M.debug_types := b
 let set_debug_combine b = M.debug_combine := b
 let set_debug_proof b = M.debug_proof := b
 let set_debug_split b = M.debug_split := b
-let set_debug_matching b = M.debug_matching := b
+let set_debug_matching i = M.debug_matching := i
+let set_debug_explanations b = M.debug_explanations := b
 
 (** additional setters *)
 let set_type_only b = M.type_only := b
@@ -306,7 +352,6 @@ let set_verbose b = M.verbose := b
 let set_greedy b = M.greedy := b
 let set_triggers_var b = M.triggers_var := b
 let set_nb_triggers b = M.nb_triggers := b
-let set_select b = M.select := b
 let set_rm_eq_existential b = M.rm_eq_existential := b
 let set_no_Ematching b = M.no_Ematching := b
 let set_nocontracongru b = M.nocontracongru := b
@@ -314,13 +359,16 @@ let set_term_like_pp b = M.term_like_pp := b
 let set_all_models b = M.all_models := b
 let set_model b = M.model := b
 let set_complete_model b = M.complete_model := b
+let set_interpretation b = M.interpretation := b
 let set_max_split b = M.max_split := b
+let set_fm_cross_limit b = M.fm_cross_limit := b
 let set_rewriting b = M.rewriting := b
 let set_proof b = M.proof := b
 let set_rules b = M.rules := b
 let set_restricted b = M.restricted := b
 let set_bottom_classes b = M.bottom_classes := b
 let set_timelimit b = M.timelimit := b
+let set_model_timelimit b = M.timelimit := b
 let set_timers b = M.timers := b
 
 let set_profiling f b =
@@ -340,6 +388,7 @@ let set_save_used_context b = M.save_used_context := b
 let debug () = !M.debug
 let debug_warnings () = !M.debug_warnings
 let debug_cc () = !M.debug_cc
+let debug_gc () = !M.debug_gc
 let debug_use () = !M.debug_use
 let debug_uf () = !M.debug_uf
 let debug_fm () = !M.debug_fm
@@ -357,6 +406,7 @@ let debug_combine () = !M.debug_combine
 let debug_proof () = !M.debug_proof
 let debug_split () = !M.debug_split
 let debug_matching () = !M.debug_matching
+let debug_explanations () = !M.debug_explanations
 
 (** additional getters *)
 let js_mode () = !M.js_mode
@@ -372,25 +422,35 @@ let verbose () = !M.verbose
 let greedy () = !M.greedy
 let triggers_var () = !M.triggers_var
 let nb_triggers () = !M.nb_triggers
-let select () = !M.select
 let rm_eq_existential () = !M.rm_eq_existential
 let no_Ematching () = !M.no_Ematching
+let no_backjumping () = !M.no_backjumping
+let no_NLA () = !M.no_NLA
+let no_ac () = !M.no_ac
 let nocontracongru () = !M.nocontracongru
 let term_like_pp () = !M.term_like_pp
+let cumulative_time_profiling () = !M.cumulative_time_profiling
 let all_models () = !M.all_models
 let model () = !M.model || !M.complete_model
+let interpretation () = !M.interpretation
+let debug_interpretation () = !M.debug_interpretation
 let complete_model () = !M.complete_model
 let max_split () = !M.max_split
+let fm_cross_limit () = !M.fm_cross_limit
 let rewriting () = !M.rewriting
 let proof () = !M.proof || !M.save_used_context || !M.debug_proof
 let rules () = !M.rules
 let restricted () = !M.restricted
 let bottom_classes () = !M.bottom_classes
 let timelimit () = !M.timelimit
+let interpretation_timelimit () = !M.interpretation_timelimit
 let enable_assertions () = !M.enable_assertions
 let profiling () =  !M.profiling
 let profiling_period () = !M.profiling_period
 let timers () = !M.timers || !M.profiling
+let case_split_policy () = !M.case_split_policy
+let instantiate_after_backjump () = !M.instantiate_after_backjump
+
 
 let replay () = !M.replay
 let replay_used_context () = !M.replay_used_context
@@ -405,7 +465,6 @@ let inequalities_plugin () = !M.inequalities_plugin
 let profiling_plugin () = !M.profiling_plugin
 let normalize_instances () = !M.normalize_instances
 let partial_bmodel () = !M.partial_bmodel
-let backward_compat () = !M.backward_compat
 
 (** particular getters : functions that are immediately executed **************)
 let exec_thread_yield () = !M.thread_yield ()
@@ -428,7 +487,7 @@ module Time = struct
   let value () =
     MyUnix.cur_time() -. !u
 
-  let set_timeout () = MyUnix.set_timeout (timelimit ())
+  let set_timeout tm = MyUnix.set_timeout tm
 
   let unset_timeout () =
     if timelimit() <> 0. then
@@ -440,3 +499,14 @@ end
 let cs_steps_cpt = ref 0
 let cs_steps () = !cs_steps_cpt
 let incr_cs_steps () = incr cs_steps_cpt
+
+
+(** open Options in every module to hide polymorphic versions of Pervasives **)
+let (<>) (a: int) (b: int) = a <> b
+let (=)  (a: int) (b: int) = a = b
+let (<)  (a: int) (b: int) = a < b
+let (>)  (a: int) (b: int) = a > b
+let (<=) (a: int) (b: int) = a <= b
+let (>=) (a: int) (b: int) = a >= b
+
+let compare  (a: int) (b: int) = Pervasives.compare a b
