@@ -7,13 +7,13 @@
 (* WARNING: a "cut" is performed on the following file in the Makefile.
    DO NOT CHANGE its format *)
 
-let version="1.01"
+let version="1.30"
 
 let release_commit = "(not released)"
 
 let release_date   = "(not released)"
 
 
-let version="1.01"
-let release_commit = "afced17160dd4c46b4b4e817b150e5c2d95ca671"
-let release_date = "Tue Feb 16 11:53:41 CET 2016"
+let version="1.30"
+let release_commit = "0447785ef027702c0cd50a62b86fb28dd54acc08"
+let release_date = "Mon Nov 21 07:54:45 CET 2016"
